import { Component, AfterViewChecked, HostListener, OnDestroy } from '@angular/core';
import { SearchService } from '../shared-services/search.service';
import { SettingsService } from '../shared-services/settings.service';
import { CatchAllListsService } from '../shared-services/catch-all-lists.service';
import { Subscription } from 'rxjs/Subscription';
import { Router } from '@angular/router';

declare var jQuery: any;

@Component({
  selector: 'app-home',
  templateUrl: './home.component.html',
  styleUrls: ['./home.component.css'],
  host: {
    '(window:resize)': 'onResize($event)'
  }
})

export class HomeComponent implements AfterViewChecked, OnDestroy {
  public initialResizePerformed = false;
  public splitterOptions: IgSplitter;
  public id = 'ResultsDetailsSplitter';
  public searchCompleteSubscription: Subscription;

  constructor(public router: Router, public searchService: SearchService, public settingsService: SettingsService,
    public catchAllListsService: CatchAllListsService) {
    this.searchCompleteSubscription = this.searchService.getSearchCompleteSubscription()
      .subscribe(message => { this.initialResizePerformed = false; });

    this.splitterOptions = {
      height: '100px',
      orientation: 'vertical',
      panels: [
        { min: 250, collapsed: false, collapsible: false },
        { min: 250, collapsed: false, collapsible: false }
      ]
    };
  }

  ngOnDestroy() {
    // Prevent memory leaks.
    this.searchCompleteSubscription.unsubscribe();
  }

  ngAfterViewChecked(): void {
    if (this.searchService.resultsList.length > 0 && !this.initialResizePerformed) {
      this.initialResizePerformed = true;
      this.resizeSplitterAndGrid();
    }
  }

  onRequestChangeClick() {
    this.router.navigateByUrl('/change');
  }

  onExportClick() {
    this.searchService.doExport(this.settingsService.filterSettings, this.settingsService.isBasic);
  }

  isRequestChangeValid() {
    // An CMF must be selected.
    if (this.settingsService.selectedCMF === null) {
      // The item is not selected.
      return false;
    } else if (!(this.catchAllListsService.supportedPartnerFuncList.indexOf(this.settingsService.selectedCMF.CUST_ACCT_GRP_CD) > -1)) {
      // The selected item does not have one of the supported partner functions.
      return false;
    }
    // TODO::TJE check if user has change permission.

    return true;
  }

  public onResize(event) {
    this.resizeSplitterAndGrid();
  }

  public resizeSplitterAndGrid() {
    // Element Height = Viewport height - element.offset.top - desired bottom margin
    const splitter: IgSplitter = jQuery('#ResultsDetailsSplitter');
    const grid: IgGrid = jQuery('#ResultsIgGrid');
    const resultsHeader: any = jQuery('#resultsHeader');

    if (splitter && splitter.offset() && grid) {
      const elementOffset: number = splitter.offset().top;
      const bottomMargin: number = jQuery('#footerContents').height();
      const viewportHeight: number = jQuery(window).height();
      const desiredSplitterHeight: number = viewportHeight - elementOffset - bottomMargin - 25;
      splitter.igSplitter({ height: desiredSplitterHeight });
      const desiredGridHeight: number = desiredSplitterHeight - resultsHeader.height() - 14;
      grid.igGrid({ height: desiredGridHeight });
    }
  }
}
