import { Component, OnInit } from '@angular/core';
import { SettingsService } from '../shared-services/settings.service';
import { CatchAllListsService } from '../shared-services/catch-all-lists.service';
import { RequestChangeService } from '../shared-services/request-change.service';
import { CHANGETYPEMAPPING } from '../shared-types/change-type-mapping';
import { CHECKABLEOBJECT } from '../shared-types/change-type-mapping';

@Component({
  selector: 'app-request-change',
  templateUrl: './request-change.component.html',
  styleUrls: ['./request-change.component.css']
})
export class RequestChangeComponent implements OnInit {
  activeChangeTypeMapping: CHANGETYPEMAPPING;

  constructor(public settingsService: SettingsService, public catchAllListsService: CatchAllListsService, 
    public requestChangeService: RequestChangeService) { }

  ngOnInit() {
    this.activeChangeTypeMapping = this.catchAllListsService.changeTypeMappingList
      .find(x => x.PartnerFunction === this.settingsService.selectedCMF.CUST_ACCT_GRP_CD);
    this.requestChangeService.sharepointChangeRequestFormUrl = null;
  }

  isAtLeastOneChangeTypeSelected(): boolean {
    const firstSelectedChangeType = this.activeChangeTypeMapping.ChangeTypeList.find(x => x.IsChecked === true);

    return !firstSelectedChangeType;
  }

  onChangeTypeSelected(selectedItem: CHECKABLEOBJECT) {
    // Enforce exclusivity.
    if (selectedItem.IsExclusive) {
      // Uncheck all items that may be checked, except for the one that was selected.
      this.activeChangeTypeMapping.ChangeTypeList.forEach(function (currItem) {
        if (currItem.DisplayString !== selectedItem.DisplayString) {
          currItem.IsChecked = false;
        }
      });
    } else {
      // Uncheck any exclusive items that may be checked.
      this.activeChangeTypeMapping.ChangeTypeList.forEach(function (currItem) {
        if (currItem.IsExclusive) {
          currItem.IsChecked = false;
        }
      });
    }
  }

  onSubmitClick() {
    // Get the selected change types.
    const selectedChangeTypes = this.activeChangeTypeMapping.ChangeTypeList
      .filter(function (item) { return item.IsChecked === true; })
      .map(function (item) { return item.DisplayString; });

    this.requestChangeService.doRequestChange(this.settingsService.selectedCMF, selectedChangeTypes);
  }

  onClickHereClick() {
    this.requestChangeService.openSharepointChangeRequestForm();
  }
}
