import { Component, OnInit, AfterViewInit, OnDestroy } from '@angular/core';
import { SearchService } from '../shared-services/search.service';
import { SettingsService } from '../shared-services/settings.service';
import { Observable } from 'rxjs/Observable';
import { Subscription } from 'rxjs/Subscription';
import { CMF } from '../shared-types/cmf';

declare var jQuery: any;

@Component({
	selector: 'app-results',
	templateUrl: './results.component.html',
	styleUrls: ['./results.component.css']
})

export class ResultsComponent implements AfterViewInit, OnDestroy {
	public gridOptions: IgGrid;
	public cellClickHandler: any;
	public renderedEventHandler: any;
	public selectionEventHandler: any;
	public cmfData = '[]';
	public id = 'ResultsIgGrid';
	public searchCompleteSubscription: Subscription;

	constructor(public searchService: SearchService, public settingsService: SettingsService) {
		this.searchCompleteSubscription = this.searchService.getSearchCompleteSubscription()
			.subscribe(message => { this.selectItemInList(null); });

		this.cellClickHandler = function (ui) {
		};

		this.renderedEventHandler = function (ui) {
		};

		this.gridOptions = {
			width: '100%',
			height: '100%',
			primaryKey: 'Id',
			autoGenerateColumns: false,
			columns: [
				{ 'headerText': 'Id', 'key': 'Id', 'dataType': 'number', 'width': '0%', hidden: true },
				{ 'headerText': 'CMF Id', 'key': 'CMF_ID', 'dataType': 'string', 'width': '18%' },
				{ 'headerText': 'Customer Name', 'key': 'CUST_NM', 'dataType': 'string', 'width': '28%' },
				{ 'headerText': 'Partner Function', 'key': 'CUST_ACCT_GRP_CD', 'dataType': 'string', 'width': '18%' },
				{ 'headerText': 'Sales Org', 'key': 'SLS_ORG_CD', 'dataType': 'string', 'width': '18%' },
				{ 'headerText': 'Distribution Channel', 'key': 'DSTRB_CHNL_CD', 'dataType': 'string', 'width': '18%' }
			],
			features: [{
				name: 'Filtering'
			}, {
				name: 'Sorting'
			},
			{
				name: 'Selection',
				mode: 'row',
				rowSelectionChanged: function (ui, args) {
					if (args.selectedRows.length === 0) {
						settingsService.setSelectedCMF(null);
					}
					else {
						settingsService.setSelectedCMF(searchService.resultsList.find(x => x.Id === Number(args.row.id)));
					}
				}
			}]
		};
	}

	ngAfterViewInit() {
		this.selectItemInList(this.settingsService.selectedCMF);
	}

	ngOnDestroy() {
		// Prevent memory leaks.
		this.searchCompleteSubscription.unsubscribe();
	}

	public selectItemInList(itemToSelect: CMF) {
		if (!this.searchService.noMatches) {
			let primaryId = 0;

			if (itemToSelect === null) {
				// We'll select the first item in the list.
				primaryId = this.searchService.resultsList[0].Id;
				this.settingsService.setSelectedCMF(this.searchService.resultsList.find(x => x.Id === Number(primaryId)));
			}
			else {
				// We'll select the item that was previously selected.
				primaryId = itemToSelect.Id;
			}

			const grid: IgGrid = jQuery('#ResultsIgGrid');
			const rows = grid.igGrid('rows');
			jQuery(rows).each(function (index, el) {
				if (Number(jQuery(el).attr('data-id')) === primaryId) {
					grid.igGridSelection('selectRow', index);
				}
			});
		}
	}
}
