import { Injectable, Output, EventEmitter } from '@angular/core';
import { Headers, Http, Response, RequestOptions, URLSearchParams } from '@angular/http';
import 'rxjs/add/operator/toPromise';
import 'rxjs/Rx';
import { Observable } from 'rxjs/Observable';
import { Subject } from 'rxjs/Subject';
import { CMF } from '../shared-types/cmf';
import { BusyIndicatorService } from '../shared-services/busy-indicator.service';
import { DialogService } from '../shared-services/dialog.service';
import { CatchAllListsService } from '../shared-services/catch-all-lists.service';
import { SettingsService } from '../shared-services/settings.service';
import { shim } from 'promise.prototype.finally';
import { environment } from '../../environments/environment';

@Injectable()
export class RequestChangeService {
  sharepointChangeRequestFormUrl: string = null;

  constructor(private http: Http, private busyIndicatorService: BusyIndicatorService,
    private dialogService: DialogService, private catchAllListsService: CatchAllListsService,
    private settingsService: SettingsService) {
    shim();
  }

  doRequestChange(itemToChange: CMF, selectedChangeTypes: string[]) {
    this.busyIndicatorService.showBusyIndicator('Opening change request form, please wait...');

    const postBody = this.getPostBody(itemToChange, selectedChangeTypes);

    const headers = new Headers({
      'Accept': 'text/plain',
      'Content-Type': 'application/json'
    });

    // Set the request options.
    const requestOptions = new RequestOptions();
    requestOptions.headers = headers;
    requestOptions.withCredentials = true;

    let promise: Promise<any>;

    promise = this.http.post(environment.requestChangeWebApiUrl, postBody, requestOptions)
      .toPromise()
      .then(response => response)
      .catch((err) => {
        this.handleError(err, this.dialogService);
      })
      .finally(() => {
        this.busyIndicatorService.hideBusyIndicator();
      });

    promise.then(results => {
      if (results) {
        this.sharepointChangeRequestFormUrl = results._body;
        this.openSharepointChangeRequestForm();
      }
    });
  }

  openSharepointChangeRequestForm() {
    window.open(this.sharepointChangeRequestFormUrl);
  }

  private handleError(error: any, dialogService: DialogService): Promise<any> {
    console.log('Error Encountered In RequestChangeService');
    console.error(error);

    dialogService.showCommunicationErrorDialog();
    return Promise.reject(error.message || error);
  }

  private getPostBody(itemToChange: CMF, selectedChangeTypes: string[]): string {
    const selectedEnvironmentsDict = {
      'DatabaseEnv': this.settingsService.selectedDatabaseEnv.Id.toString(),
      'SharepointEnv': this.settingsService.selectedSharepointEnv.Id.toString()
    };

    const myJaggedArray = {
      'CmfId': itemToChange.CMF_ID,
      'DistributionChannel': itemToChange.DSTRB_CHNL_CD,
      'SalesOrg': itemToChange.SLS_ORG_CD,
      'SelectedChangeTypes': selectedChangeTypes,
      'SelectedEnvironments': selectedEnvironmentsDict
    };

    return JSON.stringify(myJaggedArray);
  }
}
